@@ -50,7 +50,7 @@
  * Needless to say, the list MUST be sorted on the language code.
  */
 /* Sacrificial define to make introspection happy. */
-#ifndef LANGUAGE(x, y, z)
+#ifndef LANGUAGE
 #define LANGUAGE(x, y, z)
 #endif
 LANGUAGE(
@@ -210,8 +210,8 @@ LANGUAGE(
 LANGUAGE(
 	 ga	/* Irish */,
 	 WP-PANG,
-	 "D'ḟuascail Íosa Úrṁac na hÓiġe Beannaiṫe pór Éaḃa agus Áḋaiṁ."
-	 /* Jesus, Son of the blessed Virgin, redeemed the seed of Eve and Adam. */
+	 "Chuaigh bé mhórshách le dlúthspád fíorfhinn trí hata mo dhea-phorcáin bhig."
+	 /* A maiden of large appetite with an intensely white, dense spade went through the hat of my good little porker. */
 	)
 LANGUAGE(
 	 gd	/* Scottish Gaelic */,
@@ -263,9 +263,9 @@ LANGUAGE(
 	)
 LANGUAGE(
 	 hr	/* Croatian */,
-	 GLASS,
-	 "Ja mogu jesti staklo i ne boli me."
-	 /* I can eat glass and it doesn't hurt me. */
+	 MISC,
+	 "Deblji krojač: zgužvah smeđ filc u tanjušni džepić."
+	 /* A fatter taylor: I’ve crumpled a brown felt in a slim pocket. */
 	)
 LANGUAGE(
 	 hu	/* Hungarian */,
