@@ -365,16 +365,16 @@ get_face_metrics (PangoFcFont      *fcfont,
 #if HB_VERSION_ATLEAST(2,5,4)
   hb_position_t position;
 
-  if (hb_ot_metrics_get_position (hb_font, HB_OT_METRICS_UNDERLINE_SIZE, &position))
+  if (hb_ot_metrics_get_position (hb_font, HB_OT_METRICS_TAG_UNDERLINE_SIZE, &position))
     metrics->underline_thickness = position;
 
-  if (hb_ot_metrics_get_position (hb_font, HB_OT_METRICS_UNDERLINE_OFFSET, &position))
+  if (hb_ot_metrics_get_position (hb_font, HB_OT_METRICS_TAG_UNDERLINE_OFFSET, &position))
     metrics->underline_position = position;
 
-  if (hb_ot_metrics_get_position (hb_font, HB_OT_METRICS_STRIKEOUT_SIZE, &position))
+  if (hb_ot_metrics_get_position (hb_font, HB_OT_METRICS_TAG_STRIKEOUT_SIZE, &position))
     metrics->strikethrough_thickness = position;
 
-  if (hb_ot_metrics_get_position (hb_font, HB_OT_METRICS_STRIKEOUT_OFFSET, &position))
+  if (hb_ot_metrics_get_position (hb_font, HB_OT_METRICS_TAG_STRIKEOUT_OFFSET, &position))
     metrics->strikethrough_position = position;
 #endif
 }
@@ -545,7 +545,7 @@ pango_fc_font_real_get_glyph (PangoFcFont *font,
  * Since: 1.4
  * Deprecated: 1.44: Use pango_font_get_hb_font() instead
  **/
-gpointer
+FT_Face
 pango_fc_font_lock_face (PangoFcFont *font)
 {
   g_return_val_if_fail (PANGO_IS_FC_FONT (font), NULL);
@@ -810,7 +810,7 @@ pango_fc_font_get_raw_extents (PangoFcFont    *fcfont,
 	  ink_rect->x = extents.x_bearing;
 	  ink_rect->width = extents.width;
 	  ink_rect->y = -extents.y_bearing;
-	  ink_rect->height = extents.height;
+	  ink_rect->height = -extents.height;
 	}
 
       if (logical_rect)
