@@ -27,6 +27,7 @@
 #include <unistd.h>
 #endif
 
+#include "config.h"
 #include <pango/pangocairo.h>
 #include "test-common.h"
 
@@ -132,7 +133,7 @@ dump_runs (PangoLayout *layout, GString *string)
   PangoLayoutRun *run;
   PangoItem *item;
   const gchar *text;
-  gint index, index2;
+  gint index;
   gboolean has_more;
   gchar *char_str;
   gint i;
@@ -142,28 +143,18 @@ dump_runs (PangoLayout *layout, GString *string)
   iter = pango_layout_get_iter (layout);
 
   has_more = TRUE;
-  index = pango_layout_iter_get_index (iter);
-  index2 = 0;
   i = 0;
   while (has_more)
     {
       run = pango_layout_iter_get_run (iter);
+      index = pango_layout_iter_get_index (iter);
       has_more = pango_layout_iter_next_run (iter);
       i++;
 
-      if (has_more)
-        {
-          index2 = pango_layout_iter_get_index (iter);
-          char_str = g_strndup (text + index, index2 - index);
-        }
-      else
-        {
-          char_str = g_strdup (text + index);
-        }
-
       if (run)
         {
           item = ((PangoGlyphItem*)run)->item;
+          char_str = g_strndup (text + item->offset, item->length);
           font = font_name (item->analysis.font);
           g_string_append_printf (string, "i=%d, index=%d, chars=%d, level=%d, gravity=%s, flags=%d, font=%s, script=%s, language=%s, '%s'\n",
                                   i, index, item->num_chars, item->analysis.level,
@@ -175,15 +166,13 @@ dump_runs (PangoLayout *layout, GString *string)
                                   char_str);
           print_attributes (item->analysis.extra_attrs, string);
           g_free (font);
+          g_free (char_str);
         }
       else
         {
           g_string_append_printf (string, "i=%d, index=%d, no run, line end\n",
                                   i, index);
         }
-
-      g_free (char_str);
-      index = index2;
     }
   pango_layout_iter_free (iter);
 }
@@ -260,14 +249,14 @@ test_file (const gchar *filename, GString *string)
   g_assert (p);
   markup = p + 1;
   *p = '\0';
+  length = strlen (markup);
 
   parse_params (contents, &width, &ellipsize_at, &ellipsize, &wrap);
 
   layout = pango_layout_new (context);
-
   desc = pango_font_description_from_string ("Cantarell 11");
   pango_layout_set_font_description (layout, desc);
-  pango_font_description_free (desc); 
+  pango_font_description_free (desc);
 
   pango_layout_set_markup (layout, markup, length);
   g_free (contents);
@@ -278,17 +267,17 @@ test_file (const gchar *filename, GString *string)
   pango_layout_set_wrap (layout, wrap);
 
   g_string_append (string, pango_layout_get_text (layout));
-  g_string_append (string, "\n---\n\n");
+  g_string_append (string, "\n--- parameters\n\n");
   g_string_append_printf (string, "wrapped: %d\n", pango_layout_is_wrapped (layout));
   g_string_append_printf (string, "ellipsized: %d\n", pango_layout_is_ellipsized (layout));
   g_string_append_printf (string, "lines: %d\n", pango_layout_get_line_count (layout));
   if (width != 0)
     g_string_append_printf (string, "width: %d\n", pango_layout_get_width (layout));
-  g_string_append (string, "\n---\n\n");
+  g_string_append (string, "\n--- attributes\n\n");
   print_attr_list (pango_layout_get_attributes (layout), string);
-  g_string_append (string, "\n---\n\n");
+  g_string_append (string, "\n--- lines\n\n");
   dump_lines (layout, string);
-  g_string_append (string, "\n---\n\n");
+  g_string_append (string, "\n--- runs\n\n");
   dump_runs (layout, string);
 
   g_object_unref (layout);
@@ -347,7 +336,7 @@ main (int argc, char *argv[])
   const gchar *name;
   gchar *path;
 
-  g_setenv ("LC_ALL", "C", TRUE);
+  g_setenv ("LC_ALL", "en_US.UTF-8", TRUE);
   setlocale (LC_ALL, "");
 
   g_test_init (&argc, &argv, NULL);
